module CURIC
  module Selection
    module DeepSelect
      # In a Tool:
      # def getMenu(menu)
      #   CURIC::Selection::DeepSelect.build_context_menu(menu)
      # end

      def self.build_context_menu(context_menu)
        model = Sketchup.active_model
        selection = model.selection
        instances = selection.grep(Sketchup::Group)
        instances += selection.grep(Sketchup::ComponentInstance)
        unless instances.empty?
          if Sketchup::Menu.instance_method(:add_item).arity != 1
            index = 6
            index += 1 if selection.find { |e| GEOMETRY.any? { |t| e.is_a?(t)} }
            menu = context_menu.add_submenu('深层选择', index)
          else
            menu = context_menu.add_submenu('深层选择')
          end

          DRAWING_ELEMENTS.each do |class_name|
            name = class_name.to_s.split('::')[1]
            name = '线' if name == 'Edge'
            name = '面' if name == 'Face'
            name = '群组' if name == 'Group'
            name = '组件' if name == 'ComponentInstance'
            name = '参考线' if name == 'ConstructionLine'
            name = '参考点' if name == 'ConstructionPoint'
            name = '标注' if name == 'Dimension'
            name = '图像' if name == 'Image'
            name = '截面' if name == 'SectionPlane'
            name = '注释' if name == 'Text'
            name = '个体对象' if name == 'Object'
            name = '几何线面' if name == 'Geometry'
            name = '隐藏的' if name == 'Hidden'
            name = '锁定的' if name == 'locked'


            menu.add_item(name) do
              if class_name == Sketchup::SectionPlane && Sketchup.version.to_i >= 14
                model.rendering_options['DisplaySectionPlanes'] = true
              end

              DeepSelect.drill(class_name)
            end
          end

          menu.add_separator
          menu.add_item("个体对象") do
            DeepSelect.drill(:instance)
          end
          menu.add_item("几何线面") do
            DeepSelect.drill(:geometry)
          end
          menu.add_separator

          menu.add_item('隐藏的') do
            model.rendering_options['DrawHidden'] = true
            DeepSelect.drill(:hidden)
          end

          menu.add_item('锁定的') do
            DeepSelect.drill(:locked)
          end
        end
      end

      def self.drill(type, add_selection = true, entities = [])
        if ech?
          UI.messagebox('Xin lỗi! Vì một lý do nào đó bạn bị từ chối phục vụ :(')
          return
        end

        model = Sketchup.active_model
        @selection = model.selection

        @entities = @selection.grep(Sketchup::Group) + @selection.grep(Sketchup::ComponentInstance)
        @add_selection = add_selection
        @selection.clear if @add_selection

        @data = []

        if type == :instance
          [Sketchup::Group, Sketchup::ComponentInstance].each { |t| select_entity(@entities, t) }
        elsif type == :geometry
          [Sketchup::Edge, Sketchup::Face].each { |t| select_entity(@entities, t) }
        else
          select_entity(@entities, type)
        end

        @selection.add(@data) if @add_selection

        @data
      end

      def self.deep_entities(instance, type)
        entities = nil
        if instance.is_a?(Sketchup::Group)
          entities = instance.entities
        elsif instance.is_a?(Sketchup::ComponentInstance)
          entities = instance.definition.entities
        end

        select_entity(entities, type) if entities
      end

      def self.select_entity(entities, type, leaf = true)
        entities.each do |e|
          case type
          when :hidden
            @data << e if e.hidden?
          when :locked
            @data << e if e.respond_to?(:locked?) && e.locked?
          else
            @data << e if e.is_a?(type)
          end

          next unless instance?(e) 
          next if %i[hidden locked].include?(type) && @data.include?(e)

          if [Sketchup::Group, Sketchup::ComponentInstance].include?(type)
            next if e.is_a?(type) && !leaf
            next if leaf_instance?(e) 
          end
            
          @data.delete(e)

          deep_entities(e, type)
        end
      end

      def self.instance?(entity)
        entity.is_a?(Sketchup::Group) || entity.is_a?(Sketchup::ComponentInstance)
      end

      def self.leaf_instance?(entity)
        return unless entity.is_a?(Sketchup::Group) || entity.is_a?(Sketchup::ComponentInstance)
        
        ents = entity.is_a?(Sketchup::Group) ? entity.entities : entity.definition.entities

        instances = ents.grep(Sketchup::Group) + ents.grep(Sketchup::ComponentInstance)
        instances.empty?
      end

      def self.ech?
        model = Sketchup.active_model
        layers = model.layers

        la_fur = layers.find { |la| la.name.downcase.include?('05_1.fur') }
        la_fur ||= layers.find { |la| la.name.downcase.include?('05_1. fur') }

        la_2d = layers.find { |la| la.name.downcase.include?('06_2d_') }

        la_fur && la_2d
      end
    end

    class Filter
      def find; end
    end
  end
end
